precision highp int;
precision highp float;
 #include <OgreUnifiedShader.h>
mat3 mat4ToMat3(mat4 m) { return mat3(m[0].xyz, m[1].xyz, m[2].xyz); }
 const int kSC_SymbolCullingDistanceFactor = 4;
 const int kSC_OSMTileCullingDistanceFactor = 4;
 const int kSC_UseOSMDataFromAncestorDiff = 1;
 const int kSC_MaxOSMLevelFromServer = 15;
  const float kSC_Skybox_MinVisibleDistance = 500000.0;
  const float kSC_Skybox_Height = 1000000.0;
  const float kSC_UserMarker_OuterCircleRadius = 12.0;
 const int kSC_MaxBoundingBoxesPerTopoLayer = 16;
 const float kSC_CameraFOV = 42.0;
 const float kSC_ElevationExaggerationMinHeightRange = 200.0;
 const float kSC_ElevationExaggerationMinHeightRangeFactor = 1.9;
 const float kSC_ElevationExaggerationMaxHeightRange = 2000.0;
 const float kSC_ElevationExaggerationMaxHeightRangeFactor = 1.15;
 const float kSC_FlatModeExaggeration = 0.1;
uniform float mercatorScale;
uniform float elevationExaggerationFactor;
vec4 upScalePoint(vec4 point, mat4 modelMatrix)
{
    vec4 scaledPoint = point;
    float offsetZ = modelMatrix[3][2];
    scaledPoint.z =
        (scaledPoint.z + offsetZ) * mercatorScale * elevationExaggerationFactor - offsetZ;
    return scaledPoint;
}
vec4 upScalePoint(vec4 vec)
{
    vec4 scaledVec = vec;
    scaledVec.z *= mercatorScale * elevationExaggerationFactor;
    return scaledVec;
}
vec3 _applyScaleOnNormal(vec3 vec, float scale)
{
    vec3 scaledVec = vec;
    scaledVec.xy *= scale;
    return normalize(scaledVec);
}
vec3 applyElevationScaleOnNormal(vec3 vec)
{
    return _applyScaleOnNormal(vec, mercatorScale * elevationExaggerationFactor);
}
vec3 applyMercatorScaleOnNormal(vec3 vec) { return _applyScaleOnNormal(vec, mercatorScale); }
float inverseLerp(float min, float max, float value) { return (value - min) / (max - min); }
float sampleMapboxCurve(float zoomLevel, float keys[2], float values[2], float base)
{
    if (zoomLevel <= keys[0])
        return values[0];
    else if (zoomLevel >= keys[1])
        return values[1];
    else
    {
        float t = base == 1.0 ? inverseLerp(keys[0], keys[1], zoomLevel)
                              : (pow(base, zoomLevel - keys[0]) - 1.0) /
                                    (pow(base, keys[1] - keys[0]) - 1.0);
        return mix(values[0], values[1], t);
    }
}
float sampleMapboxLinearCurve(float zoomLevel, float keys[4], float values[4])
{
    if (zoomLevel <= keys[0])
        return values[0];
    else if (zoomLevel <= keys[1])
        return mix(values[1 - 1], values[1], inverseLerp(keys[1 - 1], keys[1], zoomLevel));
    else if (zoomLevel <= keys[2])
        return mix(values[2 - 1], values[2], inverseLerp(keys[2 - 1], keys[2], zoomLevel));
    else if (zoomLevel <= keys[3])
        return mix(values[3 - 1], values[3], inverseLerp(keys[3 - 1], keys[3], zoomLevel));
    else
        return values[3];
}
float fieldOfView(mat4 projectionMatrix)
{
    float fov = 2.0 * atan(1.0 / projectionMatrix[1][1]);
    return fov;
}
float calculateZoomLevel(float distance, float screenHeight, mat4 projectionMatrix)
{
    float fov = fieldOfView(projectionMatrix);
    float halfFov = fov / 2.0;
    float viewSizeFactor = tan(halfFov) * 2.0;
    float verticalViewSizeMeters = distance * viewSizeFactor;
    float metersPerPixel = verticalViewSizeMeters / screenHeight;
    float kEarthTileSize = 40075016.0;
    float pixels = kEarthTileSize / metersPerPixel;
    float kTileResolution = 256.0;
    float tiles = pixels / kTileResolution;
    return log2(tiles) - 1.0;
}
uniform float width;
uniform float height;
uniform float widthUV;
uniform float heightUV;
uniform float bufferWidth;
uniform float bufferHeight;
uniform float offsetX;
uniform float offsetY;
uniform float pixelRatio;
uniform vec2 pixelSize;
uniform mat4 modelMatrix;
uniform mat4 modelViewMatrix;
uniform mat4 projectionMatrix;
attribute vec4 position;
attribute vec2 uv0;
attribute float secondary_colour;
uniform float up;
uniform float sizeMCKeys[2]; uniform float sizeMCValues[2]; uniform float sizeMCBase;
uniform float opacityMCKeys[4]; uniform float opacityMCValues[4];
varying float vOpacity;
varying vec2 vUv;
void main()
{
    vec4 scaledPosition = upScalePoint(position, modelMatrix);
    vec4 pos = modelViewMatrix * scaledPosition;
    float depth = abs(pos.z);
    float zoomLevel = calculateZoomLevel(depth, 1.0 / pixelSize.y, projectionMatrix);
    float scale = sampleMapboxCurve(zoomLevel, sizeMCKeys, sizeMCValues, sizeMCBase);
    vOpacity = sampleMapboxLinearCurve(zoomLevel, opacityMCKeys, opacityMCValues);
    vec4 sPosition = projectionMatrix * modelViewMatrix * scaledPosition;
    vec2 screenSize = vec2(bufferWidth, bufferHeight) / 2.0;
    sPosition.xy = floor(sPosition.xy * screenSize / sPosition.w) / screenSize * sPosition.w;
    vec2 screenToView = 2.0 * pixelSize;
    float sizeToScreen = 1.0 / 2.0;
    sPosition.xy += (((
                          uv0) *
                          vec2(width * widthUV, height * heightUV) +
                      2.0 * vec2(offsetX, -offsetY)) *
                     (sPosition.w * scale * sizeToScreen * screenToView));
    gl_Position = sPosition;
    gl_Position.z -= up;
    vUv.x = uv0.x > 0.0 ? 1.0 : 0.0;
    vUv.y = uv0.y < 0.0 ? 1.0 : 0.0;
    vUv.x = (vUv.x - 0.5) * widthUV + 0.5;
    vUv.y = (vUv.y - 0.5) * heightUV + 0.5;
}
